
# q-expansion of newform 4620.2.a.l, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4620
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [-1], [1], [-6], [-3], [3], [-1], [-7], [4], [-6], [-4], [-9], [0], [3], [5], [5], [-2], [-4], [-16], [2], [-3], [-9], [-7], [-6], [7], [-12], [12], [-13], [-1], [-14], [6], [-16], [18], [10], [9], [-14], [-8], [18], [0], [18], [4], [2], [8], [-12], [-18], [-15], [13], [-16], [6], [9], [-22], [4], [-2], [-12], [-5], [-7], [10], [-14], [-26], [5], [34], [12], [11], [22], [-29], [-7], [28], [-11], [6], [37], [-1], [-19], [11], [-14], [-14], [2], [24], [-14], [7], [-3], [-16], [2], [31], [4], [8], [29], [-2], [-14], [-24], [-22], [38], [3], [-31], [3], [9], [17], [-8], [8], [-23], [30], [-4], [-39], [-22], [-14], [2], [-6], [-10], [19], [-6], [-30], [30], [38], [-17], [46], [-41], [-36], [25], [31], [-44], [11], [-51], [-4], [-6], [-5], [9], [5], [17], [-2], [30], [34], [-5], [-2], [42], [-51], [-18], [-12], [54], [-30], [52], [47], [14], [36], [14], [-19], [28], [6], [-4], [-41], [-17], [7], [16], [29], [-6], [52], [22], [54], [-24], [-30], [3], [6], [-9], [53], [51], [-46], [-19], [-6], [-10], [-42], [5], [32], [3], [-1], [-30], [-26], [58], [60], [-10], [-10], [20], [-12], [6], [3], [31], [39], [51], [-35], [44], [-32], [-22], [38], [-32], [17], [30], [12], [24], [-6], [61], [-16], [-23], [-21], [7], [6], [-15], [48], [-8], [-46], [-15], [32], [25], [-19], [-12], [-22], [-36], [-10], [45], [-50], [-68], [-15], [-48], [32], [0], [-26], [-61], [26], [32], [-60], [64], [33], [26], [4], [-34], [41], [19], [-67], [10], [-32], [-22], [28], [-60], [-49], [0], [3], [-51], [-30], [-32], [-65], [10], [24], [6], [12], [-60], [40], [11], [25], [25], [56], [-28], [-74], [71], [60], [-49], [-76], [-42], [78], [24], [65], [-41], [-46], [34], [-2], [69], [-48], [62], [-64], [31], [12], [-52], [-60], [-71], [-60], [-52], [21], [-62], [10], [-8], [64], [34], [0], [-48], [-14], [-30], [28], [-82], [-36], [24], [14], [-4], [-68], [34], [34], [42], [63], [7], [48], [-81], [-66], [76], [-11], [-8], [-16], [-45], [-4], [-40], [-13], [72], [-21], [-70], [-3], [16], [-60], [42], [-6], [72], [-32], [-52], [-19], [-20], [-26], [8], [18], [-57], [-4], [-42], [-72], [-5], [-46], [70], [-52], [-44], [73], [4], [-88], [-15], [57], [-30], [-20], [24], [-48], [-23], [80], [58], [34], [-44], [93], [-24], [-46], [2], [81], [43], [-14], [-72], [-75], [-15], [-66], [-11], [-71], [-36], [40], [40], [-76], [-23], [-15], [-66], [-84], [-29], [4], [-94], [-78], [-33], [-63], [96], [-32], [65], [-4], [13], [-37], [0], [60], [-51], [74], [-16], [-35], [54], [-69], [24], [-46], [87], [9], [-21], [31], [-95], [30], [26], [35], [68], [-66], [-9], [-33], [-20], [-24], [28], [26], [-22], [-39], [-13], [-69], [-42], [-12], [60], [62], [-69], [-36], [-65], [46], [-25], [95], [34], [18]]
